"use strict";
const DriverOptionNotSetError_1 = require("./error/DriverOptionNotSetError");
/**
 * Common driver utility functions.
 */
class DriverUtils {
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Validates if all required options are set in the driver options.
     */
    static validateDriverOptions(options) {
        if (!options.host)
            throw new DriverOptionNotSetError_1.DriverOptionNotSetError("host");
        if (!options.username)
            throw new DriverOptionNotSetError_1.DriverOptionNotSetError("username");
        if (!options.database)
            throw new DriverOptionNotSetError_1.DriverOptionNotSetError("database");
    }
    /**
     * Normalizes and builds a new driver options.
     * Extracts settings from connection url and sets to the options itself.
     */
    static buildDriverOptions(options) {
        if (options.url) {
            const parsedUrl = this.parseConnectionUrl(options.url);
            const urlDriverOptions = {
                type: options.type,
                host: parsedUrl.host,
                username: parsedUrl.username,
                password: parsedUrl.password,
                port: parsedUrl.port,
                database: parsedUrl.database
            };
            return Object.assign(urlDriverOptions, options);
        }
        return Object.assign({}, options);
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Extracts connection data from the connection url.
     */
    static parseConnectionUrl(url) {
        const urlParser = require("url");
        const params = urlParser.parse(url);
        const auth = params.auth.split(":");
        return {
            host: params.hostname,
            username: auth[0],
            password: auth[1],
            port: parseInt(params.port),
            database: params.pathname.split("/")[1]
        };
    }
}
exports.DriverUtils = DriverUtils;

//# sourceMappingURL=DriverUtils.js.map
