"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const BaseEntityManager_1 = require("./BaseEntityManager");
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its methods, whatever
 * entity type are you passing.
 */
class EntityManager extends BaseEntityManager_1.BaseEntityManager {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        super(connection);
    }
    persist(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.obtainRepository(target).persist(entity);
    }
    remove(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.obtainRepository(target).remove(entity);
    }
    /**
     * Finds entities that match given conditions.
     */
    find(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.obtainRepository(entityClass).find(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.obtainRepository(entityClass).find(conditionsOrFindOptions);
        }
        else {
            return this.obtainRepository(entityClass).find();
        }
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.obtainRepository(entityClass).findAndCount(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.obtainRepository(entityClass).findAndCount(conditionsOrFindOptions);
        }
        else {
            return this.obtainRepository(entityClass).findAndCount();
        }
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.obtainRepository(entityClass).findOne(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.obtainRepository(entityClass).findOne(conditionsOrFindOptions);
        }
        else {
            return this.obtainRepository(entityClass).findOne();
        }
    }
    /**
     * Finds entity with given id.
     */
    findOneById(entityClass, id, options) {
        return this.obtainRepository(entityClass).findOneById(id, options);
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const dbConnection = yield this.connection.driver.retrieveDatabaseConnection();
            const result = yield this.connection.driver.query(dbConnection, query);
            yield this.connection.driver.releaseDatabaseConnection(dbConnection);
            return result;
        });
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const dbConnection = yield this.connection.driver.retrieveDatabaseConnection();
            let runInTransactionResult;
            return this.connection.driver
                .beginTransaction(dbConnection)
                .then(() => runInTransaction())
                .then(result => {
                runInTransactionResult = result;
                return this.connection.driver.commitTransaction(dbConnection);
            })
                .then(() => this.connection.driver.releaseDatabaseConnection(dbConnection))
                .then(() => runInTransactionResult);
        });
    }
}
exports.EntityManager = EntityManager;

//# sourceMappingURL=EntityManager.js.map
