/*!
 */
"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
const ConnectionManager_1 = require("./connection/ConnectionManager");
const MetadataArgsStorage_1 = require("./metadata-args/MetadataArgsStorage");
const container_1 = require("./container");
// -------------------------------------------------------------------------
// Commonly Used exports
// -------------------------------------------------------------------------
__export(require("./container"));
__export(require("./decorator/columns/Column"));
__export(require("./decorator/columns/PrimaryColumn"));
__export(require("./decorator/columns/CreateDateColumn"));
__export(require("./decorator/columns/UpdateDateColumn"));
__export(require("./decorator/relations/RelationCount"));
__export(require("./decorator/indices/Index"));
__export(require("./decorator/indices/CompositeIndex"));
__export(require("./decorator/listeners/EventSubscriber"));
__export(require("./decorator/listeners/AfterInsert"));
__export(require("./decorator/listeners/BeforeInsert"));
__export(require("./decorator/listeners/AfterUpdate"));
__export(require("./decorator/listeners/BeforeUpdate"));
__export(require("./decorator/listeners/AfterRemove"));
__export(require("./decorator/listeners/BeforeRemove"));
__export(require("./decorator/relations/OneToOne"));
__export(require("./decorator/relations/OneToMany"));
__export(require("./decorator/relations/ManyToOne"));
__export(require("./decorator/relations/ManyToMany"));
__export(require("./decorator/relations/JoinTable"));
__export(require("./decorator/relations/JoinColumn"));
__export(require("./decorator/relations/RelationCount"));
__export(require("./decorator/relations/RelationId"));
__export(require("./decorator/tables/Table"));
__export(require("./decorator/tables/AbstractTable"));
__export(require("./decorator/tree/TreeLevelColumn"));
__export(require("./decorator/tree/TreeParent"));
var Connection_1 = require("./connection/Connection");
exports.Connection = Connection_1.Connection;
var ConnectionManager_2 = require("./connection/ConnectionManager");
exports.ConnectionManager = ConnectionManager_2.ConnectionManager;
var MysqlDriver_1 = require("./driver/MysqlDriver");
exports.MysqlDriver = MysqlDriver_1.MysqlDriver;
var QueryBuilder_1 = require("./query-builder/QueryBuilder");
exports.QueryBuilder = QueryBuilder_1.QueryBuilder;
var EntityManager_1 = require("./entity-manager/EntityManager");
exports.EntityManager = EntityManager_1.EntityManager;
var Repository_1 = require("./repository/Repository");
exports.Repository = Repository_1.Repository;
// -------------------------------------------------------------------------
// Commonly used functionality
// -------------------------------------------------------------------------
/**
 * Gets metadata args storage.
 */
function getMetadataArgsStorage() {
    return container_1.getFromContainer(MetadataArgsStorage_1.MetadataArgsStorage);
}
exports.getMetadataArgsStorage = getMetadataArgsStorage;
/**
 * Gets a ConnectionManager which creates connections.
 */
function getConnectionManager() {
    return container_1.getFromContainer(ConnectionManager_1.ConnectionManager);
}
exports.getConnectionManager = getConnectionManager;
/**
 * Allows to quickly create a connection based on the given options. Uses ConnectionManager.
 */
function createConnection(options) {
    return getConnectionManager().createAndConnect(options);
}
exports.createConnection = createConnection;

//# sourceMappingURL=index.js.map
