"use strict";
const TargetMetadata_1 = require("./TargetMetadata");
/**
 * Index metadata contains all information about table's index.
 */
class IndexMetadata extends TargetMetadata_1.TargetMetadata {
    // ---------------------------------------------------------------------
    // Constructor
    // ---------------------------------------------------------------------
    constructor(args) {
        super(args.target);
        this._columns = args.columns;
        this._name = args.name;
        this.isUnique = args.unique;
    }
    // ---------------------------------------------------------------------
    // Accessors
    // ---------------------------------------------------------------------
    /**
     * Gets index's name.
     */
    get name() {
        return this.entityMetadata.namingStrategy.indexName(this._name, this.columns);
    }
    /**
     * Gets the column names which are in this index.
     */
    get columns() {
        // if columns already an array of string then simply return it
        if (this._columns instanceof Array)
            return this._columns;
        // if columns is a function that returns array of field names then execute it and get columns names from it 
        const propertiesMap = this.entityMetadata.createPropertiesMap();
        return this._columns(propertiesMap).map((i) => String(i));
    }
}
exports.IndexMetadata = IndexMetadata;

//# sourceMappingURL=IndexMetadata.js.map
