"use strict";
const _ = require("lodash");
/**
 * Naming strategy that is used by default.
 */
class DefaultNamingStrategy {
    tableName(className, customName) {
        return customName ? customName : _.snakeCase(className);
    }
    columnName(propertyName, customName) {
        return customName ? customName : propertyName;
    }
    embeddedColumnName(embeddedPropertyName, columnPropertyName, columnCustomName) {
        return embeddedPropertyName + "_" + (columnCustomName ? columnCustomName : columnPropertyName);
    }
    relationName(propertyName) {
        return propertyName;
    }
    indexName(name, columns) {
        if (name)
            return name;
        return "ind_" + columns.join("_");
    }
    joinColumnInverseSideName(joinColumnName, propertyName) {
        if (joinColumnName)
            return joinColumnName;
        return propertyName;
    }
    joinTableName(firstTableName, secondTableName, firstPropertyName, secondPropertyName, firstColumnName, secondColumnName) {
        return _.snakeCase(firstTableName + "_" + firstPropertyName + "_" + secondTableName + "_" + secondColumnName);
    }
    joinTableColumnName(tableName, columnName, secondTableName, secondColumnName) {
        const column1 = tableName + "_" + columnName;
        const column2 = secondTableName + "_" + secondColumnName;
        return column1 === column2 ? column1 + "_1" : column1;
    }
    joinTableInverseColumnName(tableName, columnName, secondTableName, secondColumnName) {
        const column1 = tableName + "_" + columnName;
        const column2 = secondTableName + "_" + secondColumnName;
        return column1 === column2 ? column1 + "_2" : column1;
    }
    closureJunctionTableName(tableName) {
        return tableName + "_closure";
    }
    foreignKeyName(tableName, columnNames, referencedTableName, referencedColumnNames) {
        const key = `${tableName}_${columnNames.join("_")}_${referencedTableName}_${referencedColumnNames.join("_")}`;
        return "fk_" + require("sha1")(key); // todo: use crypto instead?
    }
}
exports.DefaultNamingStrategy = DefaultNamingStrategy;

//# sourceMappingURL=DefaultNamingStrategy.js.map
