import { Alias } from "./alias/Alias";
import { Broadcaster } from "../subscriber/Broadcaster";
import { EntityMetadataCollection } from "../metadata-args/collection/EntityMetadataCollection";
import { Driver } from "../driver/Driver";
import { EntityMetadata } from "../metadata/EntityMetadata";
import { ObjectLiteral } from "../common/ObjectLiteral";
import { DatabaseConnection } from "../driver/DatabaseConnection";
export interface JoinRelationId {
    alias: Alias;
    type: "LEFT" | "INNER";
    conditionType: "ON" | "WITH";
    condition?: string;
    mapToProperty?: string;
}
export interface RelationCountMeta {
    alias: Alias;
    conditionType: "ON" | "WITH";
    condition?: string;
    mapToProperty?: string;
    entities: {
        entity: any;
        metadata: EntityMetadata;
    }[];
}
export declare class QueryBuilder<Entity> {
    private driver;
    private entityMetadatas;
    private broadcaster;
    private dbConnection?;
    private aliasMap;
    private type;
    private selects;
    private fromEntity;
    private fromTableName;
    private fromTableAlias;
    private updateQuerySet;
    private joins;
    private joinRelationIds;
    private relationCountMetas;
    private groupBys;
    private wheres;
    private havings;
    private orderBys;
    private parameters;
    private limit;
    private offset;
    private firstResult;
    private maxResults;
    constructor(driver: Driver, entityMetadatas: EntityMetadataCollection, broadcaster: Broadcaster, dbConnection?: DatabaseConnection);
    readonly alias: string;
    delete(entity?: Function): this;
    delete(tableName?: string): this;
    update(updateSet: Object): this;
    update(entity: Function, updateSet: Object): this;
    update(tableName: string, updateSet: Object): this;
    select(selection?: string): this;
    select(selection?: string[]): this;
    select(...selection: string[]): this;
    addSelect(selection: string): this;
    addSelect(selection: string[]): this;
    addSelect(...selection: string[]): this;
    from(tableName: string, alias: string): this;
    from(entity: Function, alias: string): this;
    from(entity: Function | string, alias: string): this;
    countRelationAndMap(mapProperty: string, property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    countRelation(property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinRelationIdAndMap(mapToProperty: string, property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinRelationIdAndMap(mapToProperty: string, property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinRelationId(property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinRelationId(property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndMapMany(mapToProperty: string, property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndMapMany(mapToProperty: string, entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndMapOne(mapToProperty: string, property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndMapOne(mapToProperty: string, entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndSelect(property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoinAndSelect(entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoin(property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    innerJoin(entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndMapMany(mapToProperty: string, property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndMapMany(mapToProperty: string, entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndMapOne(mapToProperty: string, property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndMapOne(mapToProperty: string, entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndSelect(property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoinAndSelect(entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoin(property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    leftJoin(entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    where(where: string, parameters?: ObjectLiteral): this;
    andWhere(where: string, parameters?: ObjectLiteral): this;
    orWhere(where: string, parameters?: ObjectLiteral): this;
    groupBy(groupBy: string): this;
    addGroupBy(groupBy: string): this;
    having(having: string, parameters?: ObjectLiteral): this;
    andHaving(having: string, parameters?: ObjectLiteral): this;
    orHaving(having: string, parameters?: ObjectLiteral): this;
    orderBy(sort: string, order?: "ASC" | "DESC"): this;
    addOrderBy(sort: string, order?: "ASC" | "DESC"): this;
    setLimit(limit: number): this;
    setOffset(offset: number): this;
    setFirstResult(firstResult: number): this;
    setMaxResults(maxResults: number): this;
    setParameter(key: string, value: any): this;
    setParameters(parameters: ObjectLiteral): this;
    addParameters(parameters: ObjectLiteral): this;
    getSql(): string;
    getSqlWithParameters(): [string, any[]];
    execute(): Promise<any>;
    getScalarResults<T>(): Promise<T[]>;
    getSingleScalarResult<T>(): Promise<T>;
    getResults(): Promise<Entity[]>;
    getResultsAndScalarResults(): Promise<{
        entities: Entity[];
        scalarResults: any[];
    }>;
    private loadRelationCounts(dbConnection, results);
    getSingleResult(): Promise<Entity>;
    getCount(): Promise<number>;
    getResultsAndCount(): Promise<[Entity[], number]>;
    clone(options?: {
        dbConnection?: DatabaseConnection;
        skipOrderBys?: boolean;
        skipLimit?: boolean;
        skipOffset?: boolean;
    }): QueryBuilder<{}>;
    protected rawResultsToEntities(results: any[]): any[];
    protected createSelectExpression(): string;
    protected createWhereExpression(): string;
    /**
     * Replaces all entity's propertyName to name in the given statement.
     */
    private replacePropertyNames(statement);
    protected createJoinRelationIdsExpression(): string[];
    protected createJoinExpression(): string;
    protected createGroupByExpression(): string;
    protected createHavingExpression(): string;
    protected createOrderByExpression(): string;
    protected createLimitExpression(): string;
    protected createOffsetExpression(): string;
    private extractJoinMappings();
    protected join(joinType: "INNER" | "LEFT", property: string, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral, mapToProperty?: string, isMappingMany?: boolean): this;
    protected join(joinType: "INNER" | "LEFT", entity: Function, alias: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral, mapToProperty?: string, isMappingMany?: boolean): this;
    protected join(joinType: "INNER" | "LEFT", entityOrProperty: Function | string, alias: string, conditionType: "ON" | "WITH", condition: string, parameters?: ObjectLiteral, mapToProperty?: string, isMappingMany?: boolean): this;
    protected joinRelationId(joinType: "LEFT" | "INNER", mapToProperty: string | undefined, property: string, conditionType?: "ON" | "WITH", condition?: string, parameters?: ObjectLiteral): this;
    private isValueSimpleString(str);
    private isPropertyAlias(str);
}
